@@ -32,7 +32,10 @@ var _ Provider = (*GitLabProvider)(nil)
 
 // NewGitLabProvider initiates a new GitLabProvider
 func NewGitLabProvider(p *ProviderData, opts options.GitLabOptions) (*GitLabProvider, error) {
-	p.ProviderName = gitlabProviderName
+	p.setProviderDefaults(providerDefaults{
+		name: gitlabProviderName,
+	})
+
 	if p.Scope == "" {
 		p.Scope = gitlabDefaultScope
 	}
